"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SmartAccountV1Provider = void 0;
const elliptic_1 = require("elliptic");
const viem_1 = require("viem");
const abi_1 = require("./abi");
const addresses_1 = require("./addresses");
class SmartAccountV1Provider {
    constructor(publicClient, factory, sessionKeyManager, dex) {
        Object.defineProperty(this, "SESSION_MANAGER_DOMAIN_NAME", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: "LongDefi Session Key Manager"
        });
        Object.defineProperty(this, "ecdsa", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new elliptic_1.ec("secp256k1")
        });
        Object.defineProperty(this, "publicClient", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "dex", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "factory", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "sessionKeyManager", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.publicClient = publicClient;
        if (dex) {
            this.dex = dex;
        }
        else {
            const dexChain = addresses_1.DEX_CHAINS[publicClient.chain.id];
            if (!dexChain) {
                throw new Error("Dex not supported on this chain");
            }
            this.dex = dexChain;
        }
        const smartAccountV1Addresses = addresses_1.SMART_ACCOUNT_V1_CHAINS[publicClient.chain.id];
        if (factory) {
            this.factory = factory;
        }
        else {
            if (!smartAccountV1Addresses) {
                throw new Error("Smart account V1 not supported on this chain");
            }
            this.factory = smartAccountV1Addresses.smartAccountFactoryV1;
        }
        if (sessionKeyManager) {
            this.sessionKeyManager = sessionKeyManager;
        }
        else {
            if (!smartAccountV1Addresses) {
                throw new Error("Smart account V1 not supported on this chain");
            }
            this.sessionKeyManager = smartAccountV1Addresses.sessionKeyManager;
        }
    }
    async createSessionKeyRequest(owner, smartAccountSalt) {
        const sessionKeyPair = this.ecdsa.genKeyPair();
        const publicKey = sessionKeyPair.getPublic("hex");
        const kc = (0, viem_1.keccak256)(`0x${publicKey}`).slice(2);
        const sessionKeyAddress = `0x${kc.slice(-40)}`;
        const domain = {
            name: this.SESSION_MANAGER_DOMAIN_NAME,
            version: "1",
            chainId: this.publicClient.chain.id,
            verifyingContract: this.sessionKeyManager,
        };
        const types = {
            Permit: [
                {
                    name: "owner",
                    type: "address",
                },
                {
                    name: "salt",
                    type: "uint256",
                },
                {
                    name: "sessionKey",
                    type: "address",
                },
                {
                    name: "nonce",
                    type: "uint256",
                },
            ],
        };
        const isAuthorized = await this.publicClient.readContract({
            abi: abi_1.SESSION_KEY_MANAGER_ABI,
            address: this.sessionKeyManager,
            functionName: "isAuthorized",
            args: [owner, sessionKeyAddress],
        });
        if (isAuthorized) {
            throw new Error("Session key already authorized");
        }
        const nonce = await this.publicClient.readContract({
            abi: abi_1.SESSION_KEY_MANAGER_ABI,
            address: this.sessionKeyManager,
            functionName: "nonces",
            args: [owner],
        });
        const request = {
            domain,
            types,
            primaryType: "Permit",
            message: {
                owner,
                salt: smartAccountSalt,
                sessionKey: sessionKeyAddress,
                nonce,
            },
        };
        return {
            sessionKey: {
                privateKey: sessionKeyPair.getPrivate("hex"),
                address: sessionKeyAddress,
            },
            request,
        };
    }
    async revokeSessionKey(owner, smartAccountSalt, pub, enc) {
        const key = this.ecdsa.keyFromPublic(pub, enc);
        const publicKey = key.getPublic("hex");
        const kc = (0, viem_1.keccak256)(`0x${publicKey}`).slice(2);
        const sessionKey = `0x${kc.slice(-40)}`;
        const domain = {
            name: this.SESSION_MANAGER_DOMAIN_NAME,
            version: "1",
            chainId: this.publicClient.chain.id,
            verifyingContract: this.sessionKeyManager,
        };
        const types = {
            Revoke: [
                {
                    name: "owner",
                    type: "address",
                },
                {
                    name: "salt",
                    type: "uint256",
                },
                {
                    name: "sessionKey",
                    type: "address",
                },
                {
                    name: "nonce",
                    type: "uint256",
                },
            ],
        };
        const isAuthorized = await this.publicClient.readContract({
            abi: abi_1.SESSION_KEY_MANAGER_ABI,
            address: this.sessionKeyManager,
            functionName: "isAuthorized",
            args: [owner, sessionKey],
        });
        if (isAuthorized) {
            throw new Error("Session key already authorized");
        }
        const nonce = await this.publicClient.readContract({
            abi: abi_1.SESSION_KEY_MANAGER_ABI,
            address: this.sessionKeyManager,
            functionName: "nonces",
            args: [owner],
        });
        const request = {
            domain,
            types,
            primaryType: "Revoke",
            message: {
                owner,
                salt: smartAccountSalt,
                sessionKey,
                nonce,
            },
        };
        return request;
    }
    async createSwapRequest(createSwapRequestInput) {
        if (!createSwapRequestInput.smartAccount &&
            !createSwapRequestInput.initSmartAccountInput) {
            throw new Error("Must provide either `smartAccount` or `initSmartAccountInput`");
        }
        if (!this.dex) {
            throw new Error("Dex not supported");
        }
        const dexName = createSwapRequestInput.dex;
        const dex = this.dex[dexName];
        if (!dex) {
            throw new Error(`${dexName} not supported`);
        }
        let smartAccount = createSwapRequestInput.smartAccount;
        let initCode = "0x";
        const smartAccountFactory = this.factory;
        const initSmartAccountInput = createSwapRequestInput.initSmartAccountInput;
        if (initSmartAccountInput) {
            const { owner, salt } = initSmartAccountInput;
            const address = await this.publicClient.readContract({
                abi: abi_1.SMART_ACCOUNT_V1_FACTORY_ABI,
                address: smartAccountFactory,
                functionName: "getAddress",
                args: [owner, salt],
            });
            const bytecode = await this.publicClient.getBytecode({ address });
            if (bytecode && bytecode !== "0x") {
                throw new Error("Smart account already exists");
            }
            const initCodeCallData = (0, viem_1.encodeFunctionData)({
                abi: abi_1.SMART_ACCOUNT_V1_FACTORY_ABI,
                functionName: "createAccount",
                args: [owner, salt],
            });
            initCode = (smartAccountFactory.toLowerCase() +
                initCodeCallData.slice(2));
            smartAccount = address.toLowerCase();
        }
        if (!smartAccount) {
            throw new Error("Smart account not found");
        }
        const { tokenIn, tokenOut, fee, recipient, deadline, amountIn, amountOutMinimum, sqrtPriceLimitX96, } = createSwapRequestInput.swapInput;
        const approveCallData = (0, viem_1.encodeFunctionData)({
            abi: abi_1.ERC20_ABI,
            functionName: "approve",
            args: [dex.swapRouter, amountIn],
        });
        const exactInputSingleCallData = (0, viem_1.encodeFunctionData)({
            abi: abi_1.UNISWAP_V3_SWAP_ROUTER_ABI,
            functionName: "exactInputSingle",
            args: [
                {
                    tokenIn,
                    tokenOut,
                    fee,
                    recipient: recipient || smartAccount,
                    deadline,
                    amountIn,
                    amountOutMinimum,
                    sqrtPriceLimitX96: sqrtPriceLimitX96 || BigInt(0),
                },
            ],
        });
        const dest = [tokenIn, dex.swapRouter];
        const value = [];
        const func = [approveCallData, exactInputSingleCallData];
        const executeBatchCallData = (0, viem_1.encodeFunctionData)({
            abi: abi_1.SMART_ACCOUNT_V1_ABI,
            functionName: "executeBatch",
            args: [dest, value, func],
        });
        const nonce = await this.getNonceForSmartAccountV1(smartAccount, dex.factory, [tokenIn, tokenOut], fee);
        const callGasLimit = await this.calculateExecuteBatchGasLimit(smartAccount, dest, value, func);
        const verificationGasLimit = await this.calculateVerificationGasLimitV1(initCode !== "0x");
        const preVerificationGas = BigInt(0);
        let maxFeePerGas = BigInt(0);
        let maxPriorityFeePerGas = BigInt(0);
        if (!createSwapRequestInput.gasless) {
            maxFeePerGas = BigInt(30e9);
            maxPriorityFeePerGas = BigInt(2e9);
        }
        const userOpWithoutSign = {
            sender: smartAccount,
            nonce,
            initCode,
            callData: executeBatchCallData,
            callGasLimit,
            verificationGasLimit,
            preVerificationGas,
            maxFeePerGas,
            maxPriorityFeePerGas,
            paymasterAndData: "0x",
            signature: "0x",
        };
        const userOpHash = await this.publicClient.readContract({
            abi: abi_1.ENTRYPOINT_ABI,
            address: addresses_1.ENTRYPOINT_ADDRESS,
            functionName: "getUserOpHash",
            args: [userOpWithoutSign],
        });
        return { smartAccount, userOpHash, request: userOpWithoutSign };
    }
    async getNonceForSmartAccountV1(smartAccount, dexFactory, tokens, fee) {
        if (tokens.length !== 2) {
            throw new Error("Invalid token pair");
        }
        const poolAddr = await this.publicClient.readContract({
            abi: abi_1.UNISWAP_V3_FACTORY_ABI,
            address: dexFactory,
            functionName: "getPool",
            args: [tokens[0], tokens[1], fee],
        });
        const formattedPoolAddr = poolAddr.toLowerCase();
        const key = BigInt(formattedPoolAddr);
        const nonce = await this.publicClient.readContract({
            abi: abi_1.ENTRYPOINT_ABI,
            address: addresses_1.ENTRYPOINT_ADDRESS,
            functionName: "getNonce",
            args: [smartAccount, key],
        });
        return nonce;
    }
    async calculateVerificationGasLimitV1(initSmartAccount) {
        if (initSmartAccount) {
            return BigInt(300000);
        }
        else {
            return BigInt(50000);
        }
    }
    async calculateExecuteBatchGasLimit(smartAccount, dest, value, func) {
        try {
            const estimatedGas = await this.publicClient.estimateContractGas({
                abi: abi_1.SMART_ACCOUNT_V1_ABI,
                address: smartAccount,
                functionName: "executeBatch",
                args: [dest, value, func],
                account: addresses_1.ENTRYPOINT_ADDRESS,
            });
            const baseGas = BigInt(21000);
            if (estimatedGas < 30000) {
                return BigInt(500000);
            }
            return estimatedGas - baseGas;
        }
        catch (error) {
            return BigInt(200000);
        }
    }
}
exports.SmartAccountV1Provider = SmartAccountV1Provider;
//# sourceMappingURL=smartAccountV1Provider.js.map